require 'net/http'
require 'cgi'
require 'uri'

begin
  require 'json'
rescue LoadError
  require File.join File.dirname(File.dirname(__FILE__)), 'vendor', 'json.rb'
end

module Pasting
  extend self

  VERSION = '0.0.5'

  PASTING_API_URL   = URI.encode("http://api.pasting.io/".strip)

  PASTING_BASE_PATH = "/"

  USER_AGENT        = "pasting/#{VERSION} (Net::HTTP, #{RUBY_DESCRIPTION})"

  # Exception tag for errors raised while pasting.
  module Error;
    def self.exception(*args)
      RuntimeError.new(*args).extend(self)
    end
  end

  # helper module for authentication token actions
  module AuthTokenFile
    def self.filename
        File.expand_path "~/.pastingKey"
    end

    def self.read
      File.read(filename).chomp
    end

    def self.write(token)
      File.open(filename, 'w', 0600) do |f|
        f.write token
      end
    end
  end

  # auth token for authentication
  #
  # @return [String] string value of access token or `nil`, if not found
  def auth_token
    @token ||= AuthTokenFile.read rescue nil
  end

  # 
  def pasting_post(post_uri, body)
    uri = URI.parse(api_url)
    http = Net::HTTP.new(uri.host, uri.port)
    request = Net::HTTP::Post.new(post_uri)
    request.add_field('Content-Type', 'application/json')
    request.body = body.to_json
    response = http.request(request)
    response
  end

  # Create token to pasting
  # @see http://pasting.io/
  def login
    print "Pasting username: "
    username =  $stdin.gets.strip
    print "Pasting password: "
    password = $stdin.gets.strip
    body = {:username => username, :pwd => password}
    response = pasting_post('/createConsoleKey', body)
    if Net::HTTPSuccess === response
      json = JSON.parse(response.body)
      if json['st'] == 'error'
        puts json['msg']
      elsif json['st'] == 'ok'
        AuthTokenFile.write(json['console_key'])
        puts "Login Success"
      end
    else
      puts "Fail to register"
    end
  end

  # Upload a paste to http://pasting.io
  #
  # @param [String] content the code you'd like to paste
  #
  # @see http://pasting.io/
  def paste(content, options = {})
    if options[:private]
      prv = 1
    else
      prv = 0
    end 
    body = {:consoleKey => auth_token, :text => content, :protected => prv }
    uri = URI.parse(api_url)
    http = Net::HTTP.new(uri.host, uri.port)
    request = Net::HTTP::Post.new("/createFromConsole")
    request.add_field('Content-Type', 'application/json')
    request.body = body.to_json
    response = http.request(request)

    if Net::HTTPSuccess === response
      messsage = on_success(response.body, options)
      puts "#{messsage}"
      begin
        IO.popen('pbcopy', 'r+') { |clip| clip.print messsage }
      rescue 
        puts
      end
    else
      puts "Fail to register, please check user and password"
    end
  end


  # Called after an HTTP response to gist to perform post-processing.
  #
  # @param [String] body  the text body from the github api
  # @param [Hash] options  more detailed options, see
  #   the documentation for {multi_gist}
  def on_success(body, options={})
    json = JSON.parse(body)
    output = "http://pasting.io/p/#{json["documentId"]}"
    if options[:private] 
      output = output + "/#{json["hash"]}"
    end
    output
  end

  # Get the API base path
  def base_path
    PASTING_BASE_PATH
  end

  # Get the API URL
  def api_url
    PASTING_API_URL
  end

end